<?php
/**
 * FreshBooking Wordpress Plugin
 *
 * @package           FreshBooking
 * @author            Freshcase I/S
 * @copyright         2023 Freshcase
 *
 * @wordpress-plugin
 * Plugin Name:       FreshBooking for Octopus PMS
 * Author:            Freshcase I/S
 * Author URI:        https://freshbooking.dk/
 * Plugin URI:        https://freshbooking.dk/
 * Description:       FreshBooking for Octopus PMS for WordPress.
 * Version:           3.3.3
 * Text Domain:       freshcase-octopus
 * Domain Path:       /languages
 */

require_once __DIR__ . '/vendor/autoload.php';
require_once (ABSPATH . '/wp-admin/includes/file.php');
WP_Filesystem();

$dotenv = Dotenv\Dotenv::createImmutable(__DIR__);
$dotenv->safeLoad();

global $plugin_version, $backendURL;
$plugin_version = '3.3.3';

if (defined('FRESHBOOKING_STAGING')) {
   $backendURL = 'https://stage.api.freshbooking.dk';
} else {
   $backendURL = 'https://octopus.freshcase.dk';
}

// If this file is called directly, abort.
if (!defined('WPINC')) {
	die;
}

if ($_ENV['DEVMODE'] === 'true') {
   $backendURL = 'http://freshbooking_backend_dev';
}

require_once __DIR__ . '/includes/helpers.php';

function add_type_attribute($tag, $handle, $src) {
   // if not your script, do nothing and return original $tag
   if ( 'your-script-handle' !== $handle ) {
      return $tag;
   }
   // change the script tag by adding type="module" and return it.
   $tag = '<script type="module" src="' . esc_url( $src ) . '"></script>';
   return $tag;
}

function AfterPluginUpdate($upgrader_object, $options): void {
	if (
      $options['action'] == 'update' &&
      $options['type'] === 'plugin'
   )  {
		delete_transient('upgrade_freshcase-octopus');
	}
}

function PushUpdate($transient) {
   global $plugin_version;

   // Trying to get from cache first
	if (false == $remote = get_transient('upgrade_freshcase-octopus')) {
		$remote = wp_remote_get('https://repo.freshcase.dk/freshcase-octopus/info.json', array(
			'timeout' => 10,
			'headers' => array(
				'Accept' => 'application/json'
			)
      ));
 
		if (
         !is_wp_error($remote) &&
         isset( $remote['response']['code'] ) &&
         $remote['response']['code'] == 200 &&
         !empty( $remote['body'])
      ) {
			set_transient('upgrade_freshcase-octopus', $remote, 21600);
		}
	}

   $remote = json_decode($remote['body']);

   // Your installed plugin version should be on the line below! You can obtain it dynamically of course 
   if (
      $remote && version_compare($plugin_version, $remote->version, '<') &&
      version_compare($remote->requires, get_bloginfo('version'), '<')
   ) {
      // New update available
      $res = new stdClass();
      $res->slug = 'freshcase-octopus';
      $res->plugin = 'freshcase-octopus/freshcase-octopus.php';
      $res->new_version = $remote->version;
      $res->tested = $remote->tested;
      $res->package = $remote->download_url;
      $res->icons = json_decode(json_encode($remote->icons), true);

      $transient->response[$res->plugin] = $res;
   } else {
      // No update is available.
      $res = new stdClass();
      $res->id = 'freshcase-octopus-wp-plugin';
      $res->slug = 'freshcase-octopus';
      $res->plugin = 'freshcase-octopus/freshcase-octopus.php';
      $res->new_version = $remote->version;
      $res->tested = $remote->tested;
      $res->package = $remote->download_url;
      $res->url = 'https://freshbooking.dk/';
      $res->icons = json_decode(json_encode($remote->icons), true);

      $transient->no_update[$res->plugin] = $res;
   }

   $transient->checked[$res->plugin] = $remote->version;
	
   return $transient;
}

function FetchPluginInfo($res, $action, $args) {
	// Do nothing if this is not about getting plugin information
	if ('plugin_information' !== $action) {
		return false;
	}
 
	// Do nothing if it is not our plugin
	if ('freshcase-octopus' !== $args->slug) {
		return $res;
	}
 
	// Trying to get from cache first
	if (false == $remote = get_transient('upgrade_freshcase-octopus')) {
		$remote = wp_remote_get('https://repo.freshcase.dk/freshcase-octopus/info.json', array(
			'timeout' => 10,
			'headers' => array(
				'Accept' => 'application/json'
			)
      ));
 
		if (
         !is_wp_error($remote) &&
         isset( $remote['response']['code'] ) &&
         $remote['response']['code'] == 200 &&
         !empty( $remote['body'])
      ) {
			set_transient('upgrade_freshcase-octopus', $remote, 21600);
		}
 
	}
 
	if (
      !is_wp_error($remote) &&
      isset($remote['response']['code']) &&
      $remote['response']['code'] == 200 &&
      !empty($remote['body'])
   ) {
		$remote = json_decode($remote['body']);
		$res = new stdClass();
 
		$res->name = $remote->name;
		$res->slug = 'freshcase-octopus';
      $res->plugin_name = 'freshcase-octopus';
		$res->version = $remote->version;
		$res->tested = $remote->tested;
		$res->requires = $remote->requires;
		$res->author = $remote->author;
		$res->contributors = json_decode(json_encode($remote->contributors), true);
      $res->download_link = $remote->download_url;
		$res->homepage = $remote->homepage;
		$res->trunk = $remote->download_url;
		$res->requires_php = '5.3';
		$res->last_updated = $remote->last_updated;
		$res->sections = array(
			'description' => $remote->sections->description,
			'changelog' => $remote->sections->changelog,
         'screenshots' => $remote->sections->screenshots
		);
      $res->banners = array(
			'low' => $remote->banners->low,
			'high' => $remote->banners->high
		);
      $res->icons = json_decode(json_encode($remote->icons), true);
 
		return $res;
	}
	return false;
}

function PluginUpdateCheck(): void {
   global $plugin_version;
   $installedVersion = get_site_option('freshcase-octopus_version');

   if (version_compare($installedVersion, '3.0.0', '<')) {
      AddColumnToCredentialsTable();
   }

   if (version_compare($installedVersion, '3.1.0', '<')) {
      ChangesForVersion_3_1_0();
   }

   if (version_compare($installedVersion, '3.2.4', '<')) {
      ChangesForVersion_3_2_4();
   }

   if (version_compare($installedVersion, '3.2.10', '<')) {
      ChangesForVersion_3_2_10();
   }

   if (version_compare($installedVersion, '3.3.0', '<')) {
      ChangesForVersion_3_3_0();
   }

   update_option('freshcase-octopus_version', $plugin_version);
}

add_action('plugins_loaded', 'PluginUpdateCheck');

function AddCredentialsTable(): void {
   global $wpdb;

   $tableName = $wpdb->prefix . 'freshbooking_credentials';

   if ($wpdb->get_var( "show tables like '$tableName'" ) != $tableName) {

      $sql =  "CREATE TABLE `". $tableName . "` ( ";
      $sql .= "  `id` INT UNSIGNED NOT NULL auto_increment, ";
      $sql .= "  `username` VARCHAR(100) NOT NULL,";
      $sql .= "  `token` LONGTEXT NOT NULL,";
      $sql .= "  `stripe_customer_id` VARCHAR(100) NOT NULL,";
      $sql .= "  PRIMARY KEY (`id`)";
      $sql .= ") ENGINE=MyISAM DEFAULT CHARSET=latin1 AUTO_INCREMENT=1 ; ";

      require_once( ABSPATH . '/wp-admin/includes/upgrade.php' );
      dbDelta($sql);
   }
}

function ChangesForVersion_3_3_0(): void {
   global $wpdb;

   $tableName = $wpdb->prefix . 'freshcase_octopus_translations';
   $results = $wpdb->get_results("SELECT * FROM {$tableName}");

   foreach ($results as &$value) {
      $data = json_decode($value->{'message'}, true);
   
      if ($value->{'iso'} === 'da') {
         $data['hotel.addExtraRoom'] = 'Tilføj ekstra værelse';
         $data['hotel.invalidConfig'] = 'Ikke muligt!';

         $wpdb->update($tableName, array('message' => json_encode($data)), array('iso' => 'da'));
      }

      if ($value->{'iso'} === 'en') {
         $data['hotel.addExtraRoom'] = 'Add extra room';
         $data['hotel.invalidConfig'] = 'Not available!';

         $wpdb->update($tableName, array('message' => json_encode($data)), array('iso' => 'en'));
      }
   }
}

function ChangesForVersion_3_2_10(): void {
   global $wpdb;

   $tableName = $wpdb->prefix . 'freshcase_octopus_translations';
   $results = $wpdb->get_results("SELECT * FROM {$tableName}");

   foreach ($results as &$value) {
      $data = json_decode($value->{'message'}, true);
   
      if ($value->{'iso'} === 'da') {
         $data['general.termsAndConditionsNotice'] = 'Ved at klikke {button}, acceptere du følgende {terms}.';
         $data['general.termsAndConditions'] = 'Handelsbetingelser';

         $wpdb->update($tableName, array('message' => json_encode($data)), array('iso' => 'da'));
      }

      if ($value->{'iso'} === 'en') {
         $data['general.termsAndConditionsNotice'] = 'By clicking {button}, you accept the following {terms}.';
         $data['general.termsAndConditions'] = 'Terms and Conditions';

         $wpdb->update($tableName, array('message' => json_encode($data)), array('iso' => 'en'));
      }
   }
}

function ChangesForVersion_3_2_4(): void {
   global $wpdb;

   $tableName = $wpdb->prefix . 'freshcase_octopus_translations';
   $results = $wpdb->get_results("SELECT * FROM {$tableName}");

   foreach ($results as &$value) {
      $data = json_decode($value->{'message'}, true);
   
      if ($value->{'iso'} === 'da') {
         $data['booking.reservation.pleasure'] = 'Fornøjelse';
         $data['booking.reservation.bleasure'] = 'Forretning & Fornøjelse';

         $wpdb->update($tableName, array('message' => json_encode($data)), array('iso' => 'da'));
      }
   }
}

function ChangesForVersion_3_1_0(): void {
   global $wpdb;

   $tableName = $wpdb->prefix . 'freshcase_octopus_translations';
   $results = $wpdb->get_results("SELECT * FROM {$tableName}");

   foreach ($results as &$value) {
      $data = json_decode($value->{'message'}, true);
   
      if ($value->{'iso'} === 'da') {
         $data['general.perAdultPerNight'] = 'per voksen per nat';
         $data['general.perChildPerNight'] = 'per barn per nat';
         $data['general.perNight'] = 'per nat';
         $data['general.perPerson'] = 'per person';
         $data['general.perUnit'] = 'per enhed';
         $data['alert.error.discountCodeValidation'] = 'Rabatkoden blev ikke valideret!';

         $wpdb->update($tableName, array('message' => json_encode($data)), array('iso' => 'da'));
      }

      if ($value->{'iso'} === 'en') {
         $data['general.perAdultPerNight'] = 'per adult per night';
         $data['general.perChildPerNight'] = 'per child per night';
         $data['general.perNight'] = 'per night';
         $data['general.perPerson'] = 'per person';
         $data['general.perUnit'] = 'per unit';
         $data['alert.error.discountCodeValidation'] = 'Discount code couldn\'t be validated!';

         $wpdb->update($tableName, array('message' => json_encode($data)), array('iso' => 'en'));
      }
   }
}

function AddColumnToCredentialsTable() {
   global $wpdb;

   $databaseName = $wpdb->dbname;
   $tableName = $wpdb->prefix . 'freshbooking_credentials';
   $columnName = 'stripe_customer_id';

   $row = $wpdb->get_results("SELECT `COLUMN_NAME` FROM `INFORMATION_SCHEMA`.`COLUMNS` WHERE `table_name` = '{$tableName}' AND `TABLE_SCHEMA` = '{$databaseName}' AND `COLUMN_NAME` = '{$columnName}'"  );

   if (empty($row)) {
      $wpdb->query("ALTER TABLE `{$tableName}` ADD `{$columnName}` VARCHAR(100) NOT NULL");
   }
}

function GetTranslatedURL($id): string {
   if (function_exists('pll_get_post_translations')) {
      $currentLanguage = pll_current_language();
      $defaultLanguage = pll_default_language();

      $translations = pll_get_post_translations($id);

      if (!empty($translations)) {
         if (!empty($translations[$currentLanguage])) {
            $id = $translations[$currentLanguage];
         } else if (!empty($translations[$defaultLanguage])) {
            $id = $translations[$defaultLanguage];
         }
      }
   } else if ($wpmlCurrentLang = apply_filters('wpml_current_language', NULL)) {
      $id = apply_filters('wpml_object_id', $id, 'page', true, $wpmlCurrentLang);
   }

   return get_permalink($id);
}

/**
 * Register frontend scripts.
 *
 * @return void
 */
function RegisterFrontendScripts(): void {
   SetStyle();
   RegisterTargetScript('availability', ['polyfill']);
   RegisterTargetScript('hotel', ['polyfill', 'reepay', 'easy-payment']);
   RegisterTargetScript('navigation', ['polyfill']);
   RegisterTargetScript('gift-card', ['polyfill', 'reepay', 'easy-payment']);
   RegisterTargetScript('gift-card-validation', ['polyfill']);
   RegisterTargetScript('event', ['polyfill', 'reepay', 'easy-payment']);
   RegisterTargetScript('polyfill');
   wp_register_script(
      'easy-payment',
      'https://checkout.dibspayment.eu/v1/checkout.js?v=1',
      []
   );
   wp_register_script(
      'reepay',
      'https://checkout.reepay.com/checkout.js',
      []
   );
}

function GetLicenseToken(): string {
   $token = FetchAllCredentials()->get_data();

   if ($token) {
      $token = $token[0]->token;
   } else {
      $token = 'no-license';
   }

   return $token;
}

function CreateShortcode($nameSpace, $atts, $settings) {
   ob_start();

   $output = '';
   $token = GetLicenseToken();

   if ($token !== 'no-license') {
      $output .= '<div id="fresh-booking-' . $nameSpace . '-app">';
      $output .= '<' . $nameSpace . ' ';
      foreach ($atts as $key => $value) {
         $output .= $key . '="' . esc_html($value) . '" ';
      }
      $output .= ':wp-settings="' . htmlspecialchars(json_encode($settings)) . '" ';
      $output .= ':wp-translations="' . htmlspecialchars(json_encode(FetchAllTranslations()->get_data())) . '" ';
      $output .= 'wp-fresh-booking-token="' . $token . '" ';
      $output .= '/>';
      $output .= '</div>';
   } else {
      $output .= 'No FreshBooking token provided. Please contact the site administrator!';
   }

   echo $output;

   EnqueueTargetScript($nameSpace);

   return ob_get_clean();
}

/**
 * Availability form shortcode.
 *
 * @param Array $atts
 */
function AvailabilityFormShortcode($atts) {
   return CreateShortcode(
      'availability',
      shortcode_atts(
         [
            'accent-color' => '#e7c55e',
            'button-background' => '#e7c55e',
            'button-border' => null,
            'button-border-radius' => '4px',
            'button-color' => '#000000',
            'button-font-family' => 'inherit',
            'button-font-size' => '14px',
            'button-font-weight' => '400',
            'button-text-transform' => 'uppercase',
            'font-color' => '#666',
            'font-family' => 'inherit',
            'font-size' => '16px',
            'header-color' => '#000000',
            'header-font-size' => '42px',
            'input-background' => '#fff',
            'input-border' => '1px solid rgba(0, 0, 0, 0.15)',
            'input-border-radius' => '0px',
            'input-font-color' => '#000000',
            'label-font-weight' => 'bold',
            'label-padding' => '6px',
            'panel-background' => 'rgba(255, 255, 255, 0.8)',
            'panel-border' => null,
            'panel-border-radius' => '8px',
            'panel-padding' => '24px',
            'panel-outer-padding' => '24px'
         ],
      $atts),
      [
         'maximumNightsAvailable' => get_option('maximum_nights_available'),
         'maxChildAmount' => get_option('max_child_amount'),
         'maxAdultAmount' => get_option('max_adult_amount'),
         'firstSelectableDateOffset' => get_option('first_selectable_date_offset'),
         'lastSelectableDateOffset' => get_option('last_selectable_date_offset'),
         'disableAdultSelection' => get_option('disable_adult_selection'),
         'disableChildSelection' => get_option('disable_child_selection'),
         'bookingUri' => GetTranslatedURL(get_option('booking_redirect_uri')),
         'hideTitle' => get_option('hide_availability_title')
      ]
   );
}

/**
 * Hotel form shortcode.
 *
 * @param Array $atts
 */
function HotelFormShortcode($atts) {
   return CreateShortcode(
      'hotel',
      shortcode_atts(
         array(
            'accent-color' => '#e7c55e',
            'button-background' => '#e7c55e',
            'button-border-radius' => '4px',
            'button-border' => null,
            'button-color' => '#000000',
            'button-font-family' => 'inherit',
            'button-font-size' => '14px',
            'button-font-weight' => '400',
            'button-icon-color' => '#000000',
            'button-text-transform' => 'uppercase',
            'danger-color' => '#afa997',
            'font-color' => '#666',
            'font-family' => 'inherit',
            'font-size' => '16px',
            'gray-color' => 'grey',
            'header-color' => '#000000',
            'header-layout' => 'default',
            'hide-listing-group-titles' => false,
            'input-background' => '#ffffff',
            'input-border-radius' => '0px',
            'input-border' => '1px solid rgba(0, 0, 0, 0.15)',
            'input-font-color' => '#000000',
            'label-font-weight' => 'bold',
            'label-padding' => '6px',
            'layout' => 'selective',
            'link-color' => 'inherit',
            'loading-color' => '#666',
            'packages-title-max-lines' => '1',
            'panel-background' => '#ffffff',
            'panel-border-radius' => '8px',
            'panel-border' => null,
            'panel-padding' => '24px',
            'success-color' => '#e7c55e'
      ), $atts),
      [
         'maximumNightsAvailable' => get_option('maximum_nights_available'),
         'googleApiKey' => get_option('google_api_key'),
         'enableTracking' => get_option('enable_tracking'),
         'recaptchaSiteKey' => get_option('recaptcha_site_key'),
         'maxChildAmount' => get_option('max_child_amount'),
         'maxAdultAmount' => get_option('max_adult_amount'),
         'firstSelectableDateOffset' => get_option('first_selectable_date_offset'),
         'lastSelectableDateOffset' => get_option('last_selectable_date_offset'),
         'disableAdultSelection' => get_option('disable_adult_selection'),
         'disableChildSelection' => get_option('disable_child_selection'),
         'bookingUri' => GetTranslatedURL(get_option('booking_redirect_uri')),
         'headerHeight' => get_option('header_height'),
         'hideTitle' => get_option('hide_availability_title'),
         'arrivalDefault' => get_option('arrival_default'),
         'arrivalEarliest' => get_option('arrival_earliest'),
         'arrivalLatest' => get_option('arrival_latest'),
         'departureDefault' => get_option('departure_default'),
         'departureEarliest' => get_option('departure_earliest'),
         'departureLatest' => get_option('departure_latest'),
         'disableArrivalUserSelection' => get_option('disable_arrival_user_selection'),
         'disableDepartureUserSelection' => get_option('disable_departure_user_selection'),
         'hideArrivalDepartureInputs' => get_option('hide_arrival_departure_inputs'),
         'disableNamePrefix' => get_option('disable_name_prefix'),
         'disableGiftCardVoucher' => get_option('disable_gift_card_voucher'),
         'enablePackages' => get_option('enable_packages'),
         'disablePricesFrom' => get_option('disable_prices_from'),
         'hideAvailableRoomAmount' => get_option('hide_available_room_amount'),
         'disableNewsletterSignup' => get_option('disable_newsletter_signup'),
         'disableTravelingFor' => get_option('disable_traveling_for'),
         'disableCompanyField' => get_option('disable_company_field'),
      ]
   );
}

/**
 * Button shortcode.
 *
 * @param Array $atts
 */
function NavigationShortcode($atts, $content = null) {
   $atts = shortcode_atts(
      array(
         'link' => null,
         'text-transform' => 'uppercase',
         'font-family' => 'inherit',
         'background' => '#e7c55e',
         'color' => '#000000',
         'border-radius' => '4px',
         'border' => 'none',
         'font-size' => '14px',
         'font-weight' => '400'
      ),
   $atts);

   $wpSettings = [
      'bookingUri' => GetTranslatedURL(get_option('booking_redirect_uri'))
   ];

   ob_start();
   ?>
      <div
         id="<?php echo uniqid('navigation'); ?>"
         class="fresh-booking-navigation-app"
      >
         <Navigation
            label="<?php echo esc_html($content); ?>"
            link="<?php echo esc_html( $atts['link'] ); ?>"
            text-transform="<?php echo esc_html( $atts['text-transform'] ); ?>"
            font-family="<?php echo esc_html( $atts['font-family'] ); ?>"
            background="<?php echo esc_html( $atts['background'] ); ?>"
            color="<?php echo esc_html( $atts['color'] ); ?>"
            border-radius="<?php echo esc_html( $atts['border-radius'] ); ?>"
            font-size="<?php echo esc_html( $atts['font-size'] ); ?>"
            font-weight="<?php echo esc_html( $atts['font-weight'] ); ?>"
            :wp-settings="<?php echo htmlspecialchars(json_encode($wpSettings)) ?>"
         />
      </div>
   <?php

   //wp_enqueue_script('vendor');
   wp_enqueue_script('navigation');
   wp_enqueue_style('navigation');

   return ob_get_clean();
}

/**
 * GiftCard shortcode.
 *
 * @param Array $atts
 */
function GiftCardShortcode($atts) {
   return CreateShortcode(
      'gift-card',
      shortcode_atts(
         array(
            'accent-color' => '#e7c55e',
            'button-background' => '#e7c55e',
            'button-border' => null,
            'button-border-radius' => '4px',
            'button-color' => '#000000',
            'button-font-family' => 'inherit',
            'button-font-size' => '14px',
            'button-text-transform' => 'uppercase',
            'button-font-weight' => '400',
            'font-color' => '#666',
            'font-family' => 'inherit',
            'font-size' => '16px',
            'font-weight' => '400',
            'gray-color' => 'grey',
            'header-color' => '#000000',
            'header-font-size' => '32px',
            'input-background' => '#ffffff',
            'input-border' => '1px solid rgba(0, 0, 0, 0.15)',
            'input-border-radius' => '0px',
            'input-font-color' => '#000000',
            'label-font-weight' => 'bold',
            'label-padding' => '6px',
            'link-color' => 'inherit',
            'layout' => 'minimal',
            'loading-color' => '#666',
            'panel-background' => '#ffffff',
            'panel-border' => null,
            'panel-border-radius' => '8px',
            'panel-padding' => '24px',
            'place-type' => '0',
            'success-color' => '#e7c55e'
         ),
      $atts),
      [
         'headerHeight' => get_option('header_height'),
         'recaptchaSiteKey' => get_option('recaptcha_site_key')
      ]
   );
}

/**
 * GiftCardValidation shortcode.
 *
 * @param Array $atts
 */
function GiftCardValidationShortcode($atts) {
   return CreateShortcode(
      'gift-card-validation',
      shortcode_atts(
         array(
            'accent-color' => '#e7c55e',
            'button-background' => '#e7c55e',
            'button-border' => null,
            'button-border-radius' => '4px',
            'button-color' => '#000000',
            'button-font-family' => 'inherit',
            'button-font-size' => '14px',
            'button-icon-color' => '#000000',
            'button-text-transform' => 'uppercase',
            'button-font-weight' => '400',
            'font-color' => '#666',
            'font-family' => 'inherit',
            'font-size' => '16px',
            'font-weight' => '400',
            'gray-color' => 'grey',
            'header-color' => '#000000',
            'header-font-size' => '32px',
            'input-background' => '#ffffff',
            'input-border' => '1px solid rgba(0, 0, 0, 0.15)',
            'input-border-radius' => '0px',
            'input-font-color' => '#000000',
            'label-font-weight' => 'bold',
            'label-padding' => '6px',
            'loading-color' => '#666',
            'panel-background' => '#ffffff',
            'panel-border' => null,
            'panel-border-radius' => '8px',
            'panel-padding' => '24px'
         ),
      $atts),
      [
         'recaptchaSiteKey' => get_option('recaptcha_site_key')
      ]
   );
}

/**
 * Event shortcode.
 *
 * @param Array $atts
 */
function EventShortcode($atts) {
   return CreateShortcode(
      'event',
      shortcode_atts(
         array(
            'accent-color' => '#e7c55e',
            'button-background' => '#e7c55e',
            'button-border-radius' => '4px',
            'button-border' => null,
            'button-color' => '#000000',
            'button-font-family' => 'inherit',
            'button-font-size' => '14px',
            'button-font-weight' => '400',
            'button-icon-color' => '#000000',
            'button-text-transform' => 'uppercase',
            'danger-color' => '#afa997',
            'font-color' => '#666',
            'font-family' => 'inherit',
            'font-size' => '16px',
            'gray-color' => 'grey',
            'header-color' => '#000000',
            'input-background' => '#ffffff',
            'input-border-radius' => '0px',
            'input-border' => '1px solid rgba(0, 0, 0, 0.15)',
            'input-font-color' => '#000000',
            'label-font-weight' => 'bold',
            'label-padding' => '6px',
            'link-color' => 'inherit',
            'loading-color' => '#666',
            'panel-background' => '#ffffff',
            'panel-border-radius' => '8px',
            'panel-border' => null,
            'panel-padding' => '24px',
            'success-color' => '#e7c55e'
         ),
      $atts),
      [
         'recaptchaSiteKey' => get_option('recaptcha_site_key'),
         'disableGiftCardVoucher' => get_option('disable_gift_card_voucher'),
         'disablePricesFrom' => get_option('disable_prices_from'),
         'hideEvents' => get_option('hide_events'),
         'hideEventTemplates' => get_option('hide_event_templates')
      ]
   );
}

/**
 * Initialize all frontend.
 *
 * @return void
 */
function FrontendInit(): void {
   RegisterFrontendScripts();

   add_shortcode('fresh-booking-availability', 'AvailabilityFormShortcode');
   add_shortcode('fresh-booking-hotel', 'HotelFormShortcode');
   add_shortcode('fresh-booking-navigation', 'NavigationShortcode');
   add_shortcode('fresh-booking-gift-card', 'GiftCardShortcode');
   add_shortcode('fresh-booking-gift-card-validation', 'GiftCardValidationShortcode');
   add_shortcode('fresh-booking-event', 'EventShortcode');
}

/**
 * Render admin interface for the WP dashboard.
 *
 * @return void
 */
function RenderAdminContent(): void {
   $array = array();
   $pages = get_pages('hide_empty=0');
   foreach ($pages as $page) {
      $array[$page->ID] = esc_attr($page->post_title);
   }

   $wpSettings = [
      'maximumNightsAvailable' => get_option('maximum_nights_available'),
      'googleApiKey' => get_option('google_api_key'),
      'enableTracking' => get_option('enable_tracking'),
      'recaptchaSiteKey' => get_option('recaptcha_site_key'),
      'recaptchaSecretKey' => get_option('recaptcha_secret_key'),
      'maxChildAmount' => get_option('max_child_amount'),
      'maxAdultAmount' => get_option('max_adult_amount'),
      'firstSelectableDateOffset' => get_option('first_selectable_date_offset'),
      'lastSelectableDateOffset' => get_option('last_selectable_date_offset'),
      'disableAdultSelection' => get_option('disable_adult_selection'),
      'disableChildSelection' => get_option('disable_child_selection'),
      'bookingUri' => get_option('booking_redirect_uri'),
      'headerHeight' => get_option('header_height'),
      'hideTitle' => get_option('hide_availability_title'),
      'arrivalDefault' => get_option('arrival_default'),
      'arrivalEarliest' => get_option('arrival_earliest'),
      'arrivalLatest' => get_option('arrival_latest'),
      'departureDefault' => get_option('departure_default'),
      'departureEarliest' => get_option('departure_earliest'),
      'departureLatest' => get_option('departure_latest'),
      'disableArrivalUserSelection' => get_option('disable_arrival_user_selection'),
      'disableDepartureUserSelection' => get_option('disable_departure_user_selection'),
      'hideArrivalDepartureInputs' => get_option('hide_arrival_departure_inputs'),
      'disableNamePrefix' => get_option('disable_name_prefix'),
      'disableGiftCardVoucher' => get_option('disable_gift_card_voucher'),
      'enablePackages' => get_option('enable_packages'),
      'disablePricesFrom' => get_option('disable_prices_from'),
      'hideAvailableRoomAmount' => get_option('hide_available_room_amount'),
      'hideEvents' => get_option('hide_events'),
      'hideEventTemplates' => get_option('hide_event_templates'),
      'disableNewsletterSignup' => get_option('disable_newsletter_signup'),
      'disableTravelingFor' => get_option('disable_traveling_for'),
      'disableCompanyField' => get_option('disable_company_field'),
   ];

   ?>
      <div id="fresh-booking-admin-app">
         <Admin
            :wp-settings="<?php echo htmlspecialchars(json_encode($wpSettings)) ?>"
            :wp-pages="<?php echo htmlspecialchars(json_encode($array)) ?>"
         >
            <?php settings_fields('freshcase_octopus_options'); ?>
         </Admin>
      </div>
   <?php
   
   EnqueueTargetScript('admin');
}

/**
 * Create admin menu for the WP dashboard.
 *
 * @return void
 */
function CreateAdminMenu(): void {
   add_menu_page(
      __('FreshBooking for Octopus', 'freshcase-octopus'),
      'FreshBooking',
      'manage_options',
      'fresh-booking-admin',
      'RenderAdminContent',
      'dashicons-calendar-alt',
      4
   );
}

/**
 * Register admin scripts.
 *
 * @return void
 */
function RegisterAdminScripts(): void {
   RegisterTargetScript('admin', ['polyfill']);
   RegisterTargetScript('polyfill');
   SetStyle();
   // Inject WP REST API Auth
   wp_localize_script('admin', 'freshcaseOctopusAuth', array(
      'rootapiurl' => esc_url_raw(rest_url()),
      'nonce' => wp_create_nonce('wp_rest')
   ));
}

/**
 * Register all admin settings, and set defaults for some of them.
 *
 * @return void
 */
function AdminInit(): void {
   // Global settings
   add_option('header_height', '');

   register_setting('freshcase_octopus_options', 'header_height');

   // Tracking settings
   add_option('enable_tracking', 'false');
   add_option('google_api_key', '');

   register_setting('freshcase_octopus_options', 'enable_tracking');
   register_setting('freshcase_octopus_options', 'google_api_key');

   add_option('recaptcha_site_key', '');
   add_option('recaptcha_secret_key', '');

   register_setting('freshcase_octopus_options', 'recaptcha_site_key');
   register_setting('freshcase_octopus_options', 'recaptcha_secret_key');

   // Availability options
   add_option('max_child_amount', '10');
   add_option('max_adult_amount', '10');
   add_option('first_selectable_date_offset', '1');
   add_option('last_selectable_date_offset', '365');
   add_option('disable_adult_selection', 'false');
   add_option('disable_child_selection', 'false');
   add_option('booking_redirect_uri', '');
   add_option('hide_availability_title', 'false');
   add_option('maximum_nights_available', '0');

   register_setting('freshcase_octopus_options', 'max_child_amount');
   register_setting('freshcase_octopus_options', 'max_adult_amount');
   register_setting('freshcase_octopus_options', 'first_selectable_date_offset');
   register_setting('freshcase_octopus_options', 'last_selectable_date_offset');
   register_setting('freshcase_octopus_options', 'disable_adult_selection');
   register_setting('freshcase_octopus_options', 'disable_child_selection');
   register_setting('freshcase_octopus_options', 'booking_redirect_uri');
   register_setting('freshcase_octopus_options', 'hide_availability_title');
   register_setting('freshcase_octopus_options', 'maximum_nights_available');
   
   // Hotel options
   add_option('is_currency_symbols', 'false');
   add_option('arrival_default', '15:00');
   add_option('arrival_earliest', '13:00');
   add_option('arrival_latest', '17:00');
   add_option('departure_default', '10:00');
   add_option('departure_earliest', '08:00');
   add_option('departure_latest', '12:00');
   add_option('disable_arrival_user_selection', 'false');
   add_option('disable_departure_user_selection', 'false');
   add_option('hide_arrival_departure_inputs', 'false');
   add_option('disable_name_prefix', 'false');
   add_option('disable_gift_card_voucher', 'false');
   add_option('enable_packages', 'false');
   add_option('listing_layout', 'selective');
   add_option('disable_prices_from', 'false');
   add_option('hide_available_room_amount', 'false');
   add_option('disable_newsletter_signup', 'false');
   add_option('disable_traveling_for', 'false');
   add_option('disable_company_field', 'false');

   register_setting('freshcase_octopus_options', 'is_currency_symbols');
   register_setting('freshcase_octopus_options', 'arrival_default');
   register_setting('freshcase_octopus_options', 'arrival_earliest');
   register_setting('freshcase_octopus_options', 'arrival_latest');
   register_setting('freshcase_octopus_options', 'departure_default');
   register_setting('freshcase_octopus_options', 'departure_earliest');
   register_setting('freshcase_octopus_options', 'departure_latest');
   register_setting('freshcase_octopus_options', 'disable_arrival_user_selection');
   register_setting('freshcase_octopus_options', 'disable_departure_user_selection');
   register_setting('freshcase_octopus_options', 'hide_arrival_departure_inputs');
   register_setting('freshcase_octopus_options', 'disable_name_prefix');
   register_setting('freshcase_octopus_options', 'disable_gift_card_voucher');
   register_setting('freshcase_octopus_options', 'enable_packages');
   register_setting('freshcase_octopus_options', 'listing_layout');
   register_setting('freshcase_octopus_options', 'disable_prices_from');
   register_setting('freshcase_octopus_options', 'hide_available_room_amount');
   register_setting('freshcase_octopus_options', 'disable_newsletter_signup');
   register_setting('freshcase_octopus_options', 'disable_traveling_for');
   register_setting('freshcase_octopus_options', 'disable_company_field');

   // Event options
   add_option('hide_events', 'false');
   add_option('hide_event_templates', 'false');

   register_setting('freshcase_octopus_options', 'hide_events');
   register_setting('freshcase_octopus_options', 'hide_event_templates');
}

function LoadTranslationJson(string $iso) {
   global $wp_filesystem;

   $filePath = WP_PLUGIN_DIR . '/freshcase-octopus/assets/' . $iso . '.json';
   $message = json_encode(json_decode($wp_filesystem->get_contents($filePath)));

   return $message;
}

/**
 * Create freshcase_octopus_translations table in database.
 * 
 * @return void
 */
function CreatePluginDatabase(): void {
   global $wpdb;

   $tableName = $wpdb->prefix . 'freshcase_octopus_translations';

   if ($wpdb->get_var( "show tables like '$tableName'" ) != $tableName) {

      $sql =  "CREATE TABLE `". $tableName . "` ( ";
      $sql .= "  `id` INT UNSIGNED NOT NULL auto_increment, ";
      $sql .= "  `iso` CHAR(2) NOT NULL,";
      $sql .= "  `message` LONGTEXT NOT NULL,";
      $sql .= "  PRIMARY KEY (`id`),";
      $sql .= "  UNIQUE (iso) ";
      $sql .= ") ENGINE=MyISAM DEFAULT CHARSET=latin1 AUTO_INCREMENT=1 ; ";

      require_once( ABSPATH . '/wp-admin/includes/upgrade.php' );
      dbDelta($sql);

      $wpdb->insert($tableName, array('iso' => 'da', 'message' => LoadTranslationJson('da')));
      $wpdb->insert($tableName, array('iso' => 'en', 'message' => LoadTranslationJson('en')));
   }
}

/**
 * Resets all translations (for a given iso) to the default values (if available)
 *
 * @param WP_REST_Request $data
 * @return WP_REST_Response
 */
function ResetTranslation(WP_REST_Request $data): WP_REST_Response {
   global $wpdb, $wp_filesystem;

   $tableName = $wpdb->prefix . 'freshcase_octopus_translations';
   $iso = $data['iso'];

   try {
      if ($iso) {
         $wpdb->update($tableName, array('message' => LoadTranslationJson($iso)), array('iso' => $iso));

         return new WP_REST_Response([], 200);
      }
   } catch (\Throwable $th) {
      return new WP_REST_Response([], 500);
   }
}

/**
 * Fetches all translations
 *
 * @return WP_REST_Response
 */
function FetchAllTranslations(): WP_REST_Response {
   global $wpdb;

   $tableName = $wpdb->prefix . 'freshcase_octopus_translations';

   $data = $wpdb->get_results("SELECT * FROM $tableName");

   return new WP_REST_Response($data);
}

/**
 * Fetches all Octopus credentials
 *
 * @return WP_REST_Response
 */
function FetchAllCredentials(): WP_REST_Response {
   global $wpdb;

   $tableName = $wpdb->prefix . 'freshbooking_credentials';

   $data = $wpdb->get_results("SELECT * FROM $tableName LIMIT 1");

   return new WP_REST_Response($data);
}

function UpdateTranslations(WP_REST_Request $data) {
   global $wpdb;

   $tableName = $wpdb->prefix . 'freshcase_octopus_translations';
 
   if (isset($data['translations'])) {
      $translations = $data['translations'];

      try {
         foreach ($translations as $key => $translation) {
            $doesExist = $wpdb->get_var($wpdb->prepare("SELECT COUNT(*) FROM $tableName WHERE iso = '$key' LIMIT 1"));
            
            if ($doesExist) {
               $wpdb->update($tableName, array('message' => json_encode($translation)), array('iso' => $key));
            } else {
               $wpdb->insert($tableName, array('iso' => $key, 'message' => json_encode($translation)));
            }
         }
      } catch (\Exception $ex) {
         return $ex->getMessage();
      }
   }
}

function CheckLicenseStatus(WP_REST_Request $data) {
   global $backendURL, $plugin_version;

   $response = wp_remote_get('' . $backendURL . '/api/v3/license/info', array(
      'headers' => array(
         'Content-Type' => 'application/json',
         'Accept' => 'application/json',
         'Authorization' => 'Bearer ' . $data['token'] . '',
         'Fresh-Booking-Ver' => $plugin_version
      )
   ));

   if (is_wp_error($response)) {
      $error_message = $response->get_error_message();

      return new WP_REST_Response(array(
         'success' =>  false,
         'errors' => $error_message
      ));
   }

   return new WP_REST_Response(array(
      'success' => true,
      'result' => json_decode(wp_remote_retrieve_body($response))
   ));
}

function ClearCache(WP_REST_Request $data) {
   global $backendURL, $plugin_version;

   $image = $data['image'];
   $token = $data['token'];

   $options = [
      'body' => json_encode(array(
         'image' => $image
      )),
      'headers' => [
         'Content-Type' => 'application/json',
         'Accept' => 'application/json',
         'Authorization' => 'Bearer ' . $token . '',
         'Fresh-Booking-Ver' => $plugin_version
      ],
      'method' => 'POST'
   ];

   $response = wp_remote_post('' . $backendURL . '/api/v3/license/clear-cache', $options);

   if (is_wp_error($response)) {
      $error_message = $response->get_error_message();

      return new WP_REST_Response(array(
         'success' =>  false,
         'errors' => $error_message
      ));
   }

   return new WP_REST_Response(json_decode(wp_remote_retrieve_body($response)));
}

function UpdateOctopusSettings(WP_REST_Request $data) {
   global $backendURL, $plugin_version;

   try {
      $hotelId = $data['hotelId'];
      $restaurantId = $data['restaurantId'];
      $token = $data['token'];

      if (
         (isset($hotelId) && $hotelId != "") &&
         (isset($restaurantId) && $restaurantId != "")
      ) {
         $options = [
            'body' => json_encode(array(
               'hotelId' => $hotelId,
               'restaurantId' => $restaurantId,
            )),
            'headers' => [
               'Content-Type' => 'application/json',
               'Accept' => 'application/json',
               'Authorization' => 'Bearer ' . $token . '',
               'Fresh-Booking-Ver' => $plugin_version
            ],
            'method' => 'POST'
         ];

         $response = wp_remote_post('' . $backendURL . '/api/v3/license/set-hotel-and-restaurant', $options);

         if (is_wp_error($response)) {
            $error_message = $response->get_error_message();

            return new WP_REST_Response(array(
               'success' =>  false,
               'errors' => $error_message
            ));
         }

         return new WP_REST_Response(json_decode(wp_remote_retrieve_body($response)));
      }
   } catch (\Exception $ex) {
      return new WP_REST_Response([$ex->getMessage()], 500);
   }

   return new WP_REST_Response([], 500);
}

function UpdateLicense(WP_REST_Request $data) {
   global $backendURL, $plugin_version;

   $username = $data['username'];
   $password = $data['password'];
   $stripeCustomerId = $data['stripeCustomerId'];

   if (
      (isset($username) && $username != "") &&
      (isset($password) && $password != "") &&
      (isset($stripeCustomerId) && $stripeCustomerId != "")
   ) {
      $options = [
         'body' => json_encode(array(
            'username' => $username,
            'password' => $password,
            'stripe_customer_id' => $stripeCustomerId,
         )),
         'headers' => [
            'Content-Type' => 'application/json',
            'Accept' => 'application/json',
            'Fresh-Booking-Ver' => $plugin_version
         ],
         'method' => 'POST'
      ];

      $response = wp_remote_post('' . $backendURL . '/api/v3/license/activate', $options);

      if (is_wp_error($response)) {
         $error_message = $response->get_error_message();

         return new WP_REST_Response(array(
            'success' => false,
            'errors' => $error_message
         ), 500);
      }

      if ($response['response']['code'] === 401) {
         return new WP_REST_Response([], 401);
      }

      try {
         $freshBookingToken = json_decode($response['body'])->token->plainTextToken;

         if (isset($freshBookingToken)) {
            global $wpdb;

            $tableName = $wpdb->prefix . 'freshbooking_credentials';
            $doesExist = $wpdb->get_var($wpdb->prepare("SELECT COUNT(*) FROM $tableName LIMIT 1"));

            if ($doesExist) {
               $wpdb->update($tableName, array('token' => $freshBookingToken, 'username' => $username, 'stripe_customer_id' => $stripeCustomerId ), array('ID' => 1));
            } else {
               $wpdb->insert($tableName, array('username' => $username, 'token' => $freshBookingToken, 'stripe_customer_id' => $stripeCustomerId ));
            }

            return new WP_REST_Response(['token' => FetchAllCredentials()->get_data()[0]->token]);
         }
      } catch (\Exception $ex) {
         return new WP_REST_Response([$ex->getMessage()], 500);
         // return $ex->getMessage();
      }
   } else {
      return new WP_REST_Response([], 422);
   }

   return new WP_REST_Response([], 500);
}

function DeleteTranslation(WP_REST_Request $data): void {
   global $wpdb;

   $tableName = $wpdb->prefix . 'freshcase_octopus_translations';

   $wpdb->delete($tableName, array( 'iso' => $data['iso'] ));
}

function VerifyRecaptcha(WP_REST_Request $data) {
   // https://www.google.com/recaptcha/admin/create
   $response = wp_remote_post('https://www.google.com/recaptcha/api/siteverify', array(
      'headers' => array(
         'Content-Type' => 'application/x-www-form-urlencoded'
      ),
      'body' => http_build_query(array(
         'secret' => get_option('recaptcha_secret_key'),
         'response' => $data['token']
      ))
   ));

   return new WP_REST_Response(array(
      'success' => json_decode(wp_remote_retrieve_body($response))->success
   ));
}

add_filter('pre_set_site_transient_update_plugins', 'PushUpdate');
add_filter('plugins_api', 'FetchPluginInfo', 20, 3);
add_action('upgrader_process_complete', 'AfterPluginUpdate', 10, 2);

load_plugin_textdomain('freshcase-octopus', false, dirname(plugin_basename(__FILE__)) . '/languages/');

register_activation_hook(__FILE__, 'CreatePluginDatabase');
register_activation_hook(__FILE__, 'AddCredentialsTable');

add_action('init', 'FrontendInit');
add_action('admin_menu', 'CreateAdminMenu');
add_action('admin_init', 'AdminInit');
add_action('admin_enqueue_scripts', 'RegisterAdminScripts');

add_action('rest_api_init', function () {
   register_rest_route('freshcase-octopus/v1', '/translations', array(
      'methods' => 'GET',
      'callback' => 'FetchAllTranslations',
      'permission_callback' => function() {
         return true;
      }
   ));
   register_rest_route('freshcase-octopus/v1', '/credentials', array(
      'methods' => 'GET',
      'callback' => 'FetchAllCredentials',
      'permission_callback' => function() {
         return true;
      }
   ));
   register_rest_route('freshcase-octopus/v1', '/recaptcha/(?P<token>.+)', array(
      'methods' => 'GET',
      'callback' => 'VerifyRecaptcha',
      'permission_callback' => function() {
         return true;
      }
   ));
   register_rest_route('freshcase-octopus/v1', '/translations/update', array(
      'methods' => 'POST',
      'callback' => 'UpdateTranslations',
      'permission_callback' => function() {
         return current_user_can('manage_options');
      }
   ));
   register_rest_route('freshcase-octopus/v1', '/translations/reset', array(
      'methods' => 'POST',
      'callback' => 'ResetTranslation',
      'permission_callback' => function() {
         return current_user_can('manage_options');
      }
   ));
   register_rest_route('freshcase-octopus/v1', '/translations/delete/(?P<iso>\w+)', array(
      'methods' => 'DELETE',
      'callback' => 'DeleteTranslation',
      'permission_callback' => function() {
         return current_user_can('manage_options');
      }
   ));
   register_rest_route('freshcase-octopus/v1', '/license/update', array(
      'methods' => 'POST',
      'callback' => 'UpdateLicense',
      'permission_callback' => function() {
         return current_user_can('manage_options');
      }
   ));
   register_rest_route('freshcase-octopus/v1', '/license/update-settings', array(
      'methods' => 'POST',
      'callback' => 'UpdateOctopusSettings',
      'permission_callback' => function() {
         return current_user_can('manage_options');
      }
   ));
   register_rest_route('freshcase-octopus/v1', '/license/clear-cache', array(
      'methods' => 'POST',
      'callback' => 'ClearCache',
      'permission_callback' => function() {
         return current_user_can('manage_options');
      }
   ));
   register_rest_route('freshcase-octopus/v1', '/license/status/(?P<token>.+)', array(
      'methods' => 'GET',
      'callback' => 'CheckLicenseStatus',
      'permission_callback' => function() {
         return true;
      }
   ));
});

?>